@@ -1,10 +1,11 @@
 # META NAME PdExternalsSearch
 # META DESCRIPTION Search for externals zipfiles on puredata.info
 # META AUTHOR <Chris McCormick> chris@mccormick.cx
+# META AUTHOR <IOhannes m zmölnig> zmoelnig@iem.at
 # ex: set setl sw=2 sts=2 et
 
 # Search URL:
-# http://puredata.info/search_rss?SearchableText=xtrnl-
+# http://deken.puredata.info/search?name=foobar
 
 # The minimum version of TCL that allows the plugin to run
 package require Tcl 8.4
@@ -18,6 +19,37 @@ package require http 2
 package require pdwindow 0.1
 package require pd_menucommands 0.1
 
+namespace eval ::deken:: {
+    variable version
+}
+
+## only register this plugin if there isn't any newer version already registered
+## (if ::deken::version is defined and is higher than our own version)
+proc ::deken::versioncheck {version} {
+    if { [info exists ::deken::version ] } {
+        set v0 [split $::deken::version "."]
+        set v1 [split $version "."]
+        foreach x $v0 y $v1 {
+            if { $x > $y } {
+                ::pdwindow::debug "\[deken\]: installed version \[$::deken::version\] > $version...skipping!\n"
+                return 0
+            }
+            if { $x < $y } {
+                ::pdwindow::debug "\[deken\]: installed version \[$::deken::version\] < $version...overwriting!\n"
+                set ::deken::version $version
+                return 1
+            }
+        }
+        ::pdwindow::debug "\[deken\]: installed version \[$::deken::version\] == $version...skipping!\n"
+        return 0
+    }
+    set ::deken::version $version
+    return 1
+}
+
+## put the current version of this package here:
+if { [::deken::versioncheck 0.2.1] } {
+
 namespace eval ::deken:: {
     namespace export open_searchui
     variable mytoplevelref
@@ -35,14 +67,74 @@ set ::deken::statustimer ""
 
 set ::deken::backends [list]
 proc ::deken::register {fun} {
-    lappend ::deken::backends $fun
+    set ::deken::backends [linsert $::deken::backends 0 $fun]
 }
 
+proc ::deken::gettmpdir {} {
+    proc _iswdir {d} { expr [file isdirectory $d] * [file writable $d] }
+    set tmpdir ""
+    catch {set tmpdir $::env(TRASH_FOLDER)} ;# very old Macintosh. Mac OS X doesn't have this.
+    if {[_iswdir $tmpdir]} {return $tmpdir}
+    catch {set tmpdir $::env(TMP)}
+    if {[_iswdir $tmpdir]} {return $tmpdir}
+    catch {set tmpdir $::env(TEMP)}
+    if {[_iswdir $tmpdir]} {return $tmpdir}
+    set tmpdir "/tmp"
+    set tmpdir [pwd]
+    if {[_iswdir $tmpdir]} {return $tmpdir}
+}
+proc ::deken::vbs_unzipper {zipfile {path .}} {
+    ## this is w32 only
+    if { "Windows" eq "$::deken::platform(os)" } { } { return 0 }
+    if { "" eq $::deken::_vbsunzip } {
+        set ::deken::_vbsunzip [ file join [::deken::gettmpdir] unzip.vbs ]
+    }
+
+    if {[file exists $::deken::_vbsunzip]} {} {
+        ## no script yet, create one
+        set script {
+Set fso = CreateObject("Scripting.FileSystemObject")
+
+'The location of the zip file.
+ZipFile = fso.GetAbsolutePathName(WScript.Arguments.Item(0))
+'The folder the contents should be extracted to.
+ExtractTo = fso.GetAbsolutePathName(WScript.Arguments.Item(1))
+
+'If the extraction location does not exist create it.
+If NOT fso.FolderExists(ExtractTo) Then
+   fso.CreateFolder(ExtractTo)
+End If
+
+'Extract the contants of the zip file.
+set objShell = CreateObject("Shell.Application")
+set FilesInZip=objShell.NameSpace(ZipFile).items
+objShell.NameSpace(ExtractTo).CopyHere(FilesInZip)
+Set fso = Nothing
+Set objShell = Nothing
+}
+        if {![catch {set fileId [open $::deken::_vbsunzip "w"]}]} {
+            puts $fileId $script
+            close $fileId
+        }
+    }
+    if {[file exists $::deken::_vbsunzip]} {} {
+        ## still no script, give up
+        return 0
+    }
+    ## try to call the script
+    if { [ catch { exec cscript $::deken::_vbsunzip $zipfile .} stdout ] } {
+        ::pdwindow::debug "\[deken\] VBS-unzip: $::deken::_vbsunzip\n$stdout\n"
+        return 0
+    }
+    return 1
+}
+set ::deken::_vbsunzip ""
+
 proc ::deken::get_writable_dir {paths} {
     set fs [file separator]
     set access [list RDWR CREAT EXCL TRUNC]
     foreach p $paths {
-        if { [ catch { file mkdir $p } ] } {}
+        #if { [ catch { file mkdir $p } ] } {}
         for {set i 0} {True} {incr i} {
             set tmpfile "${p}${fs}dekentmp.${i}"
             if {![file exists $tmpfile]} {
@@ -111,19 +203,6 @@ proc ::deken::readconfig {paths filename} {
 
 
 ::deken::readconfig $::sys_staticpath deken-plugin.conf
-if { [ info exists ::deken::installpath ] } {
-    set ::deken::installpath [ ::deken::get_writable_dir [list $::deken::installpath ] ]
-} {
-    set ::deken::installpath [ ::deken::get_writable_dir $::sys_staticpath ]
-}
-
-# console message to let them know we're loaded
-::pdwindow::post  "deken-plugin.tcl (Pd externals search) in $::current_plugin_loadpath loaded.\n"
-if { "$::deken::installpath" == "" } {
-    ::pdwindow::error "deken: No writeable directory found in:\n"
-    foreach p $::sys_staticpath { ::pdwindow::error "\t- $p\n" }
-    ::pdwindow::error "deken: Will not be able to download/install libraries\n"
-}
 
 set ::deken::platform(os) $::tcl_platform(os)
 set ::deken::platform(machine) $::tcl_platform(machine)
@@ -137,12 +216,17 @@ if { [ string match "Windows *" "$::deken::platform(os)" ] > 0 } {
 # normalize W32 CPUs
 if { "Windows" eq "$::deken::platform(os)" } {
     # in redmond, intel only produces 32bit CPUs,...
-    if { "intel" eq "$::deken::platform(machine)" } { set ::deken::platform(machine) "i386" }
+    if { "intel" eq "$::deken::platform(machine)" } { set ::deken::platform(machine) "i686" }
     # ... and all 64bit CPUs are manufactured by amd
     #if { "amd64" eq "$::deken::platform(machine)" } { set ::deken::platform(machine) "x86_64" }
 }
 
-::pdwindow::post "Platform detected: $::deken::platform(os)-$::deken::platform(machine)-$::deken::platform(bits)bit\n"
+# console message to let them know we're loaded
+## but only if we are being called as a plugin (not as built-in)
+if { "" != "$::current_plugin_loadpath" } {
+    ::pdwindow::post "deken-plugin.tcl (Pd externals search) in $::current_plugin_loadpath loaded.\n"
+    ::pdwindow::post "Platform detected: $::deken::platform(os)-$::deken::platform(machine)-$::deken::platform(bits)bit\n"
+}
 
 # architectures that can be substituted for eachother
 array set ::deken::architecture_substitutes {}
@@ -167,10 +251,14 @@ proc ::deken::status {msg} {
         set ::deken::statustext ""
     }
 }
-
+proc ::deken::scrollup {} {
+    variable mytoplevelref
+    $mytoplevelref.results see 0.0
+}
 proc ::deken::post {msg {tag ""}} {
     variable mytoplevelref
     $mytoplevelref.results insert end "$msg\n" $tag
+    $mytoplevelref.results see end
 }
 proc ::deken::clearpost {} {
     variable mytoplevelref
@@ -189,6 +277,16 @@ proc ::deken::highlightable_posttag {tag} {
     # make sure that the 'highlight' tag is topmost
     $mytoplevelref.results tag raise highlight
 }
+proc ::deken::prompt_installdir {} {
+    set installdir [tk_chooseDirectory -title "Install libraries to directory:"]
+    if { "$installdir" != "" } {
+        set ::deken::installpath $installdir
+        return 1
+    }
+    return 0
+}
+
+
 proc ::deken::update_searchbutton {mytoplevel} {
     if { [$mytoplevel.searchbit.entry get] == "" } {
         $mytoplevel.searchbit.button configure -text [_ "Show all" ]
@@ -208,7 +306,7 @@ proc ::deken::open_searchui {mytoplevel} {
         $mytoplevel.results tag configure warn -foreground orange
         $mytoplevel.results tag configure info -foreground grey
         $mytoplevel.results tag configure highlight -foreground blue
-        $mytoplevel.results tag configure archmatch -foreground black
+        $mytoplevel.results tag configure archmatch
         $mytoplevel.results tag configure noarchmatch -foreground grey
     }
     ::deken::post "To get a list of all available externals, try an empty search." info
@@ -236,6 +334,8 @@ proc ::deken::create_dialog {mytoplevel} {
     bind $mytoplevel.searchbit.entry <Key-Return> "::deken::initiate_search $mytoplevel"
     bind $mytoplevel.searchbit.entry <KeyRelease> "::deken::update_searchbutton $mytoplevel"
     focus $mytoplevel.searchbit.entry
+    button $mytoplevel.searchbit.button -text [_ "Show all"] -default active -width 9 -command "::deken::initiate_search $mytoplevel"
+    pack $mytoplevel.searchbit.button -side right -padx 6 -pady 3
 
     frame $mytoplevel.warning
     pack $mytoplevel.warning -side top -fill x
@@ -247,9 +347,6 @@ proc ::deken::create_dialog {mytoplevel} {
     label $mytoplevel.status.label -textvariable ::deken::statustext
     pack $mytoplevel.status.label -side left -padx 6
 
-    button $mytoplevel.searchbit.button -text [_ "Show all"] -default active -width 9 -command "::deken::initiate_search $mytoplevel"
-    pack $mytoplevel.searchbit.button -side right -padx 6 -pady 3
-
     text $mytoplevel.results -takefocus 0 -cursor hand2 -height 100 -yscrollcommand "$mytoplevel.results.ys set"
     scrollbar $mytoplevel.results.ys -orient vertical -command "$mytoplevel.results yview"
     pack $mytoplevel.results.ys -side right -fill y
@@ -264,8 +361,7 @@ proc ::deken::initiate_search {mytoplevel} {
     if { [ catch {
         set results [::deken::search_for [$mytoplevel.searchbit.entry get]]
     } stdout ] } {
-        puts "online?"
-        puts "$stdout"
+        ::pdwindow::debug "\[deken\]: online? $stdout\n"
         ::deken::status "Unable to perform search. Are you online?"
     } else {
     # delete all text in the results
@@ -281,6 +377,7 @@ proc ::deken::initiate_search {mytoplevel} {
             ::deken::show_result $mytoplevel $counter $r 0
             incr counter
         }
+	::deken::scrollup
     } else {
         ::deken::post "No matching externals found. Try using the full name e.g. 'freeverb'."
     }
@@ -298,8 +395,6 @@ proc ::deken::show_result {mytoplevel counter result showmatches} {
                 ::deken::post "$title\n\t$comment\n" [list $tag $matchtag]
                 ::deken::highlightable_posttag $tag
                 ::deken::bind_posttag $tag <Enter> "+::deken::status $status"
-                # have to decode the URL here because otherwise percent signs cause tcl to bug out - not sure why - scripting languages...
-                #::deken::bind_posttag $tag <1> [list ::deken::clicked_link $mytoplevel [urldecode $URL] $filename]
                 ::deken::bind_posttag $tag <1> "$cmd"
             }
 }
@@ -307,36 +402,66 @@ proc ::deken::show_result {mytoplevel counter result showmatches} {
 # handle a clicked link
 proc ::deken::clicked_link {URL filename} {
     ## make sure that the destination path exists
-    if { "$::deken::installpath" == "" } { set ::deken::installpath [ ::deken::get_writable_dir $::sys_staticpath ] }
-    if { "$::deken::installpath" == "" } {
-        ::deken::clearpost
+    ### if ::deken::installpath is set, use the first writable item
+    ### if not, get a writable item from one of the searchpaths
+    ### if this still doesn't help, ask the user
+    set installdir ""
+    if { [ info exists ::deken::installpath ] } {
+        ## any previous choice?
+        set installdir [ ::deken::get_writable_dir [list $::deken::installpath ] ]
+    }
+    if { "$installdir" == "" } {
+        ## search the default paths
+        set installdir [ ::deken::get_writable_dir $::sys_staticpath ]
+    }
+    if { "$installdir" == "" } {
+        ## ask the user (and remember the decision)
+        ::deken::prompt_installdir
+        set installdir [ ::deken::get_writable_dir [list $::deken::installpath ] ]
+    }
+    if { "$installdir" == "" } {
+        #::deken::clearpost
         ::deken::post "No writeable directory found in:" warn
-        foreach p $::sys_staticpath { ::deken::post "\t- $p\n" warn }
-        ::deken::post "Cannot download/install libraries!\n" warn
-    } {
-    set fullpkgfile "$::deken::installpath/$filename"
+        foreach p $::sys_staticpath { ::deken::post "\t- $p" warn }
+        ::deken::post "Cannot download/install libraries!" warn
+        return
+    }
+    switch -- [tk_messageBox -message \
+                   "Install to directory $installdir?" \
+                   -type yesnocancel -default "yes" \
+                   -icon question] {
+                       no {set installdir ""
+                           if {[::deken::prompt_installdir]} {
+                               set installdir [ ::deken::get_writable_dir [list $::deken::installpath ] ] }
+                           if { "$installdir" eq "" } return}
+                       cancel return}
+
+    set fullpkgfile "$installdir/$filename"
     ::deken::clearpost
-    ::deken::post "Commencing downloading of:\n$URL\nInto $::deken::installpath..."
+    ::deken::post "Commencing downloading of:\n$URL\nInto $installdir..."
     ::deken::download_file $URL $fullpkgfile
     set PWD [ pwd ]
-    cd $::deken::installpath
+    cd $installdir
     set success 1
     if { [ string match *.zip $fullpkgfile ] } then {
-        if { [ catch { exec unzip -uo $fullpkgfile } stdout ] } {
-            puts $stdout
-            set success 0
+        if { [ ::deken::vbs_unzipper $fullpkgfile  $installdir ] } { } {
+            if { [ catch { exec unzip -uo $fullpkgfile } stdout ] } {
+                ::pdwindow::debug "$stdout\n"
+                set success 0
+            }
         }
     } elseif  { [ string match *.tar.gz $fullpkgfile ]
                 || [ string match *.tgz $fullpkgfile ]
               } then {
         if { [ catch { exec tar xzf $fullpkgfile } stdout ] } {
-            puts $stdout
+            ::pdwindow::debug "$stdout\n"
             set success 0
         }
     }
     cd $PWD
     if { $success > 0 } {
-        ::deken::post "Successfully unzipped $filename into $::deken::installpath.\n"
+        ::deken::post "Successfully unzipped $filename into $installdir.\n"
+        catch { exec rm $fullpkgfile }
     } else {
         # Open both the fullpkgfile folder and the zipfile itself
         # NOTE: in tcl 8.6 it should be possible to use the zlib interface to actually do the unzip
@@ -344,9 +469,8 @@ proc ::deken::clicked_link {URL filename} {
         ::deken::post "Please perform the following steps manually:"
         ::deken::post "1. Unzip $fullpkgfile."
         pd_menucommands::menu_openfile $fullpkgfile
-        ::deken::post "2. Copy the contents into $::deken::installpath.\n"
-        pd_menucommands::menu_openfile $::deken::installpath
-    }
+        ::deken::post "2. Copy the contents into $installdir.\n"
+        pd_menucommands::menu_openfile $installdir
     }
 }
 
@@ -435,15 +559,13 @@ proc ::deken::search_for {term} {
     return $result
 }
 
-# create an entry for our search in the "help" menu
+# create an entry for our search in the "help" menu (or re-use an existing one)
 set mymenu .menubar.help
-if {$::windowingsystem eq "aqua"} {
-    set inserthere 3
-} else {
-    set inserthere 4
+if { [catch {
+    $mymenu entryconfigure [_ "Find externals"] -command {::deken::open_searchui .externals_searchui}
+} _ ] } {
+    $mymenu add command -label [_ "Find externals"] -command {::deken::open_searchui .externals_searchui}
 }
-$mymenu insert $inserthere separator
-$mymenu insert $inserthere command -label [_ "Find externals"] -command {::deken::open_searchui .externals_searchui}
 # bind all <$::modifier-Key-s> {::deken::open_helpbrowser .helpbrowser2}
 
 # http://rosettacode.org/wiki/URL_decoding#Tcl
@@ -499,6 +621,7 @@ proc urldecode {str} {
 proc ::deken::search::puredata.info {term} {
     set searchresults [list]
 
+    set term [ join $term "&name=" ]
     set token [http::geturl "http://deken.puredata.info/search?name=$term"]
     set contents [http::data $token]
     set splitCont [split $contents "\n"]
@@ -516,8 +639,9 @@ proc ::deken::search::puredata.info {term} {
             set filename [ file tail $URL ]
             set cmd [list ::deken::clicked_link $decURL $filename]
             set pkgverarch [ ::deken::parse_filename $filename ]
+            set archs [lindex $pkgverarch 2]
 
-            set match [::deken::architecture_match [lindex $pkgverarch 2] ]
+            set match [::deken::architecture_match "$archs" ]
 
             set comment "Uploaded by $creator @ $date"
             set status $URL
@@ -530,92 +654,5 @@ proc ::deken::search::puredata.info {term} {
     return [lsort -dictionary -decreasing -index 5 $searchresults ]
 }
 
-
-## ####################################################################
-## searching apt (if available)
-namespace eval ::deken::apt {
-    namespace export search
-    namespace export install
-    variable distribution
-}
-
-if { [ catch { exec apt-cache -v } _ ] } { } {
-if { [ catch { exec lsb_release -si } ::deken::apt::distribution ] } { unset ::deken::apt::distribution }
-}
-proc ::deken::apt::search {name} {
-    set result []
-    if { [ catch { exec apt-cache madison } _ ] } {
-        ::deken::post "Unable to run 'apt-cache madison'" error
-    } {
-    set name [ string tolower $name ]
-    if { "$name" eq "gem" } {
-        set searchname $name
-    } elseif { "$name" eq "" } {
-        set searchname "^pd-.*"
-    } else {
-        set searchname "^pd-${name}.*"
-    }
-    array unset pkgs
-    array set pkgs {}
-    set io [ open "|apt-cache madison $searchname" r ]
-    while { [ gets $io line ] >= 0 } {
-        #puts $line
-        set llin [ split "$line" "|" ]
-        set pkgname [ string trim [ lindex $llin 0 ] ]
-
-        #if { $pkgname ne $searchname } { continue }
-        set ver_  [ string trim [ lindex $llin 1 ] ]
-        set info_ [ string trim [ lindex $llin 2 ] ]
-        if { "Packages" eq [ lindex $info_ end ] } {
-            set suite [ lindex $info_ 1 ]
-            set arch  [ lindex $info_ 2 ]
-            if { ! [ info exists pkgs($ver_) ] } {
-                set pkgs($ver_) [ list $pkgname $suite $arch ]
-            }
-        }
-    }
-    foreach {v inf} [ array get pkgs ] {
-        set pkgname [ lindex $inf 0 ]
-        set suite   [ lindex $inf 1 ]
-        set arch    [ lindex $inf 2 ]
-        set name $pkgname/$v
-        set cmd "::deken::apt::install ${pkgname}=$v"
-        set match 1
-        set comment "Provided by ${::deken::apt::distribution} (${suite})"
-        set status "${pkgname}_${v}_${arch}.deb"
-
-        lappend result [list $name $cmd $match $comment $status]
-    }
-    }
-    return [lsort -dictionary -decreasing -index 1 $result ]
-}
-
-proc ::deken::apt::install {pkg} {
-    if { [ catch { exec which gksudo } gsudo ] } {
-        ::deken::post "Please install 'gksudo', if you want to install system packages via deken..." error
-    } {
-        ::deken::clearpost
-        set prog "apt-get install -y --show-progress ${pkg}"
-        # for whatever reasons, we cannot have 'deken' as the description
-        # (it will always show $prog instead)
-        set desc deken::apt
-        set cmdline "$gsudo -D $desc -- $prog"
-        #puts $cmdline
-        set io [ open "|${cmdline}" ]
-        while { [ gets $io line ] >= 0 } {
-            ::deken::post "apt: $line"
-        }
-        if { [ catch { close $io } ret ] } {
-            ::deken::post "apt::install failed to install $pkg" error
-            ::deken::post "\tDid you provide the correct password and/or" error
-            ::deken::post "\tis the apt database locked by another process?" error
-            #puts stderr "::deken::apt::install ${options}"
-        }
-    }
-}
-
-# only add this backend, if we are actually running Debian or a derivative
-if { [ info exists ::deken::apt::distribution ] } {
-  ::deken::register ::deken::apt::search
-}
 ::deken::register ::deken::search::puredata.info
+}
